
//
// Demonstrates the use of DATEVLD.DLL.
// November 3, 1991.
// Copr. (c) 1991, Albert C. Schmitt.  All Rights Reserved.
//

#include <windows.h>
#include <dos.h>
#include "generic.h"
#include "datevld.h"

HANDLE hInst;
char   ach[128];

LONG IncDay(LPSTR datebuf, int inc)
{
    LONG jDate;

    jDate = toJul(datebuf) + (LONG)inc;
    toDate(datebuf, jDate);

    return jDate;
}

void SetDlgJulian(HWND hDlg, LONG jDate)
{
    // display the Julian date in dialog
    wsprintf((LPSTR)ach, "%ld", jDate);
    SetDlgItemText(hDlg, IDC_JDATE, (LPSTR)ach);

    // display day of week
    SetDlgItemText(hDlg, IDC_SDATE, (LPSTR)dayName(jDate));
}

#if defined(__BORLANDC__)
#pragma argsused
#endif
BOOL FAR PASCAL DateDlgProc(HWND hDlg,
                            unsigned message,
                            WORD wParam,
                            LONG lParam)
{
    char datebuf[12];
    LONG jDate;
#if defined(__BORLANDC__)
    struct date d;
#else
	 struct dosdate_t d;
#endif

    switch (message) {
        case WM_INITDIALOG:

#if defined(__BORLANDC__)
            getdate(&d);
            StrDate((LPSTR)datebuf, d.da_mon, d.da_day, d.da_year);
#else
			_dos_getdate(&d);
			StrDate((LPSTR)datebuf, d.month, d.day, d.year);
#endif

            SetDlgItemText(hDlg, IDC_DATE, (LPSTR)datebuf);
            SetDlgJulian(hDlg, toJul((LPSTR)datebuf));

            return TRUE;

        case WM_COMMAND:

            if (wParam == IDOK) {

                GetDlgItemText(hDlg, IDC_DATE, (LPSTR)datebuf, 11);
                jDate = toJul((LPSTR)datebuf);
                if (jDate == 0L)
                    MessageBox(hDlg, "Invalid Date", "Generic", MB_OK | MB_ICONEXCLAMATION);
                else {
                    toDate((LPSTR)datebuf, jDate);
                    SetDlgItemText(hDlg, IDC_DATE, (LPSTR)datebuf);
                }
                SetDlgJulian(hDlg, jDate);
                return TRUE;

            } else if (wParam == ID_NEXT) {

                GetDlgItemText(hDlg, IDC_DATE, (LPSTR)datebuf, 11);
                jDate = IncDay(datebuf, 1);
                SetDlgItemText(hDlg, IDC_DATE, (LPSTR)datebuf);
                SetDlgJulian(hDlg, jDate);
                return TRUE;

            } else if (wParam == ID_PRIOR) {

                GetDlgItemText(hDlg, IDC_DATE, (LPSTR)datebuf, 11);
                jDate = IncDay(datebuf, -1);
                SetDlgItemText(hDlg, IDC_DATE, (LPSTR)datebuf);
                SetDlgJulian(hDlg, jDate);
                return TRUE;

            } else if (wParam == IDCANCEL) {
                EndDialog(hDlg, TRUE);
                return TRUE;
            }

            break;
    }

    return FALSE;
}

#if defined(__BORLANDC__)
#pragma argsused
#endif
BOOL FAR PASCAL AboutDlgProc(HWND hDlg,
                             unsigned message,
                             WORD wParam,
                             LONG lParam)
{
    switch (message) {
        case WM_INITDIALOG:
            return TRUE;

        case WM_COMMAND:
            if (wParam == IDOK || wParam == IDCANCEL) {
                EndDialog(hDlg, TRUE);
                return TRUE;
            }
            break;
    }

    return FALSE;
}

long FAR PASCAL MainWndProc(HWND hWnd,
                            unsigned message,
                            WORD wParam,
                            LONG lParam)
{
    FARPROC lpProcAbout;
    FARPROC lpProcDate;

    switch (message) {

        case WM_COMMAND:

            if (wParam == IDM_ABOUT) {

                lpProcAbout = MakeProcInstance(AboutDlgProc, hInst);
                DialogBox(hInst, "AboutBox", hWnd, lpProcAbout);
                FreeProcInstance(lpProcAbout);

            } else if (wParam == IDM_DATE) {

                lpProcDate = MakeProcInstance(DateDlgProc, hInst);
                DialogBox(hInst, "DateDialog", hWnd, lpProcDate);
                FreeProcInstance(lpProcDate);

            }
            break;

        case WM_DESTROY:
            PostQuitMessage(0);
            break;

        default:
            return DefWindowProc(hWnd, message, wParam, lParam);
    }

    return 0L;
}

BOOL InitApplication(HANDLE hInstance)
{
    WNDCLASS wc;

    wc.style         = NULL;
    wc.lpfnWndProc   = MainWndProc;
    wc.cbClsExtra    = 0;
    wc.cbWndExtra    = 0;
    wc.hInstance     = hInstance;
    wc.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject (WHITE_BRUSH);
    wc.lpszMenuName  = "GenericMenu";
    wc.lpszClassName = "GenericWClass";

    return RegisterClass(&wc);
}

BOOL InitInstance(HANDLE hInstance, int nCmdShow)
{
    HWND hWnd;

    hInst = hInstance;
    hWnd = CreateWindow("GenericWClass",
                        "Date Conversion Demo",
                        WS_OVERLAPPEDWINDOW,
                        CW_USEDEFAULT,
                        CW_USEDEFAULT,
                        CW_USEDEFAULT,
                        CW_USEDEFAULT,
                        NULL,
                        NULL,
                        hInstance,
                        NULL);

    if (!hWnd)
        return FALSE;

    ShowWindow(hWnd, nCmdShow);
    UpdateWindow(hWnd);

    return TRUE;
}

#if defined(__BORLANDC__)
#pragma argsused
#endif
int PASCAL WinMain (HANDLE hInstance,
                    HANDLE hPrevInstance,
                    LPSTR lpszCmdParam,
                    int nCmdShow)
{
    MSG msg;

    if (!hPrevInstance)
        if (!InitApplication(hInstance))
            return FALSE;

    if (!InitInstance(hInstance, nCmdShow))
        return FALSE;

    while (GetMessage(&msg, NULL, 0, 0)) {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

    return msg.wParam;
}

