{
* Demonstrates the use of DATEVLD.DLL.
* November 3, 1991.
* Copr. (c) 1991, Albert C. Schmitt.  All Rights Reserved.
}

program Generic;
uses Strings, WinTypes, WinProcs, WObjects, WinDos, DateVld;

{$R GENERIC.RES}

const
        { date dialog constants }
	IDC_DATE  = 101;	{ edit field containing date }
	IDC_JDATE = 102;	{ static field: Julian representation of date }
	ID_NEXT   = 103;	{ next date button }
	ID_PRIOR  = 104;	{ prior date button }
	IDC_SDATE = 105;	{ static field: day of week }

        { menu constants }
	IDM_ABOUT = 100;
	IDM_DATE  = 101;

type
	TMyApplication = object(TApplication)
		procedure InitMainWindow; virtual;
	end;

	PMyWindow = ^TMyWindow;
	TMyWindow = object(TWindow)
		constructor Init(AParent: PWindowsObject; ATitle: PChar);
                procedure Date(var Msg: TMessage); virtual cm_first + IDM_DATE;
		procedure About(var Msg: TMessage); virtual cm_First + IDM_ABOUT;
	end;

	PDateDialog = ^TDateDialog;
        TDateDialog = object(TDialog)
		procedure SetupWindow; virtual;
		procedure SetJulian(jDate : LongInt); virtual;
		function IncDay(incr : Integer) : LongInt; virtual;
        	procedure Ok(var Msg: TMessage); virtual id_First + id_OK;
		procedure Next(var Msg: TMessage); virtual id_First + ID_NEXT;
		procedure Prior(var Msg: TMessage); virtual id_First + ID_PRIOR;
        private
        	datebuf : array [0..11] of char;
	end;

{--------------------------------------------------}
{ TMyWindow's method implementations:              }
{--------------------------------------------------}

constructor TMyWindow.Init(AParent: PWindowsObject; ATitle: PChar);
begin
	TWindow.Init(AParent, ATitle);
	Attr.Menu := LoadMenu(HInstance, 'GenericMenu');
end;

procedure TMyWindow.Date(var Msg: TMessage);
begin
	Application^.ExecDialog(New(PDateDialog, Init(@Self, 'DateDialog')));
end;

procedure TMyWindow.About(var Msg: TMessage);
begin
	Application^.ExecDialog(New(PDialog, Init(@Self, 'AboutBox')));
end;

{--------------------------------------------------}
{ TDateDialog's method implementations:            }
{--------------------------------------------------}

procedure TDateDialog.SetupWindow;
var
	y, m, d, dow : Word;
begin
	TDialog.SetupWindow;

	GetDate(y,m,d,dow);
	StrDate(datebuf, m, d, y);
	SetDlgItemText(HWindow, IDC_DATE, datebuf);
	SetJulian(toJul(datebuf));
end;

procedure TDateDialog.SetJulian(jDate : LongInt);
begin
	{ display the Julian date in dialog }
	wvsprintf(datebuf, '%ld', jDate);
	SetDlgItemText(HWindow, IDC_JDATE, datebuf);

	{ display day of week }
	SetDlgItemText(HWindow, IDC_SDATE, dayName(jDate));
end;

function TDateDialog.IncDay(incr : Integer) : LongInt;
var
	jDate : LongInt;
begin
	jDate := toJul(datebuf) + incr;
	toDate(datebuf, jDate);

	IncDay := jDate;
end;

procedure TDateDialog.Ok(var Msg: TMessage);
var
	jDate : LongInt;
begin
	GetDlgItemText(HWindow, IDC_DATE, datebuf, 11);
	jDate := toJul(datebuf);

	if (jDate = 0) then
		MessageBox(HWindow, 'Invalid Date', 'Generic', MB_OK or MB_ICONEXCLAMATION)
	else begin
		toDate(datebuf, jDate);
		SetDlgItemText(HWindow, IDC_DATE, datebuf);
	end;

	SetJulian(jDate);
end;

procedure TDateDialog.Next(var Msg: TMessage);
var
	jDate : LongInt;
begin
	GetDlgItemText(HWindow, IDC_DATE, datebuf, 11);
	jDate := IncDay(1);
	SetDlgItemText(HWindow, IDC_DATE, datebuf);
	SetJulian(jDate);
end;

procedure TDateDialog.Prior(var Msg: TMessage);
var
	jDate : LongInt;
begin
	GetDlgItemText(HWindow, IDC_DATE, datebuf, 11);
	jDate := IncDay(-1);
	SetDlgItemText(HWindow, IDC_DATE, datebuf);
	SetJulian(jDate);
end;

{--------------------------------------------------}
{ TMyApplication's method implementations:         }
{--------------------------------------------------}

procedure TMyApplication.InitMainWindow;
begin
	MainWindow := New(PMyWindow, Init(nil, 'Date Conversion Demo'));
end;

{--------------------------------------------------}
{ Main program:                                    }
{--------------------------------------------------}

var
  MyApp : TMyApplication;

begin
  MyApp.Init('GenericWClass');
  MyApp.Run;
  MyApp.Done;
end.
